#include <avr/io.h> 
#include <util/twi.h> 

#define ACK   1 
#define NACK  0 
#define WAIT_TWINT while (!(TWCR & _(TWINT)));

#define TWI_BAUD  100000

volatile uint8_t twiLastOpError; //   

//---------------------- i2c stop condition  -----------------------
void twiStop(void) {
  TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
  while(TWCR & (1<<TWSTO));
}

//----------------------- i2c start condition  ---------------------
void twiStart(uint8_t sla, uint8_t br) {

  twiLastOpError = 0;

  // Init bus, set freq
  if (br) {
    TWSR = 0;
    TWBR = br;
  }

  // START condition
  TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);

  // wait until transmission completed
  // WAIT_TWINT
  while(!(TWCR & (1<<TWINT)));

  //TWI Status Register
  if ((TW_STATUS != TW_START) && (TW_STATUS != TW_REP_START)) goto error;

  // SLA address
  TWDR = sla;
  TWCR = (1<<TWINT) | (1<<TWEN);

  // wail until transmission completed
  // WAIT_TWINT
  while(!(TWCR & (1<<TWINT)));

  // TWI Status Register
  if ( (TW_STATUS != TW_MT_SLA_ACK) && (TW_STATUS != TW_MR_SLA_ACK)) goto error;

  twiLastOpError = 0;
  return;
error:
  twiLastOpError = 1;
}

//---------------------- i2c write data  ----------------------------
void twiWrite(uint8_t dat, uint8_t useACK) {

  if (twiLastOpError != 0) return;

  // data
  TWDR = dat;
  TWCR = (1<<TWINT) | (1<<TWEN);

  // wait
  while(!(TWCR & (1<<TWINT)));

  // TWI Status Register
  if( TW_STATUS != TW_MT_DATA_ACK) twiLastOpError = 2;
}

//---------------------- i2c read data  -----------------------------
uint8_t twiRead(uint8_t useACK) {
  if (twiLastOpError != 0) return 0;
  TWCR = _(TWINT) | _(TWEN) | ((useACK)?_(TWEA):0);  // Transfer data used (or not) acknowledge
  WAIT_TWINT;
//  while (!(TWCR & _(TWINT))); // Wait until data complete
  return(TWDR); // data
}

