#define LED_ON		0x01 
#define LED_OFF		0x00 

#define LEDS_BLOCKSIZE 	16 //  
#define LEDS_BLOCK_HB 	0x8000 //    

#define LP_OE_LO	PORTD &= ~(1<<6);   // OE low - leds block enabled
#define LP_OE_HI	PORTD |= (1<<6);   // OE hi - leds block disabled
#define LP_DS_LO	PORTD &= ~(1<<5);   // DS
#define LP_DS_HI	PORTD |= (1<<5);   // DS
#define LP_SH_LO	PORTB &= ~(1<<0);   // SHCP
#define LP_SH_HI	PORTB |= (1<<0);   // SHCP
#define LP_ST_LO	PORTD &= ~(1<<7);   // STCP
#define LP_ST_HI	PORTD |= (1<<7);   // STCP

#define DELAY_200ns	asm("nop"); asm("nop"); 

//      
unsigned char leds_order[LEDS_BLOCKSIZE] = {2,1,4,3,6,5,0,7,9,8,11,10,13,12,15,14};

// bit = 1  
void leds_block(unsigned int ll) {
  unsigned char c;
  unsigned int l;
     
  //    
  l = 0;
  for (c=0 ; c < LEDS_BLOCKSIZE ; c++) if (ll & (1<<c)) l |= 1<<leds_order[c];

  for (c=0 ; c < LEDS_BLOCKSIZE ; c++) {
    LP_SH_LO;  // SHCP Low;
    if (l & LEDS_BLOCK_HB) {
      LP_DS_LO; // low ignites led
    } else {
      LP_DS_HI; // hi = led off
    }
    DELAY_200ns
    LP_SH_HI; // shcp hi
    l <<= 1;
  }

  LP_ST_HI;
  DELAY_200ns;
  LP_ST_LO;
} 

// Status LED: 1 (LED_ON) = led on, 0 (LED_OFF) = led off
void leds_status(uint8_t l){
  l = l & 1; //     - 
  if (l) PORTD &= ~_(4); else PORTD |= _(4);
}

//  
void leds_intro(void) {
  uint8_t i;

  for (i=0; i<LEDS_BLOCKSIZE; i++) {
    leds_block(1<<i);
    delay_ms(150);
  }
  leds_block(0);
}

// Init LEDs: pins i/o, reset
void leds_init(void) {

  PORTD &=  0b01011111;
  PORTD |=  0b01010000; 
  DDRD  |=  0b11110000; // D4 output - status led

  LP_OE_HI;		// OE hi -  
  DDRB  |=  0b00000001; // B0 - SCK 595 

  leds_block(0);        //  
  LP_OE_LO;		// OE low -  

  // check leds
  leds_block(0xffff);	//   
  delay_ms(200);	// 
  leds_block(0);        //   
  leds_status(LED_OFF);
}
